package com.qs.mp.pay.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.qs.mp.common.enums.PayOrderTypeEnum;
import com.qs.mp.common.utils.DateUtils;
import com.qs.mp.common.utils.LogUtil;
import com.qs.mp.common.utils.StringUtils;
import com.qs.mp.common.utils.http.OkHttpUtil;
import com.qs.mp.pay.domain.PayOrder;
import com.qs.mp.pay.service.IPayOrderService;
import com.qs.mp.pay.service.IWalletService;
import com.qs.mp.system.service.id.BizIdGenerator;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

/**
 * @auther duota
 * @create 2021 2021/9/4 1:59 下午
 * @describe
 */
@Service
public class WalletServiceImpl implements IWalletService {

  protected final Logger logger = LoggerFactory.getLogger(this.getClass().getSimpleName());

  @Autowired
  private IPayOrderService payOrderService;

  @Autowired
  private BizIdGenerator bizIdGenerator;

  @Value("${pay.notify.url}")
  private String notifyUrl;  //支付成功前端跳转地址
  @Value("${pay.call.backUrl}")
  private String callbackUrl;  //支付回调地址
  @Value("${pay.base.url}")
  private String baseUrl = "https://jlpays.kakapaypay.com/";
  @Value("${wx.appId}")
  private String appId ="wxb86cb7f459fc3675";

  @Value("${pay.shopNo}")
  private String payShopNo;  //平台商户号
  @Value("${pay.sign}")
  private String paySign;  //平台商户签名

  private static final String REQ_STATUS_SUCCESS = "R0001";
  private static final String PAY_RESOURCE = "2"; //2:微信支付
  private static final String PAY_RESOURCE_TYPE = "1017"; //2:微信小程序

  @Override
  public JSONObject merchRechargeOrderCreate(String custId,String merchId,String refType,String refId,String openId, int money) {
    String reqUrl = baseUrl+"/pay/payMoneyCTB";
    JSONObject params = new JSONObject();
    //merchid不为空，则为商户充值，如果merchid为空，则为客户充值
    //查询客户三方支付信息
      // 客户充值的商户号和签名为平台
      params = buildPayOrderReqData(payShopNo, paySign, money, openId);


    String result = OkHttpUtil.postJsonParams(reqUrl,params.toJSONString());
    logger.info("request params:"+ params.toJSONString()+"  result:"+result);
    if(StringUtils.isBlank(result)){
      LogUtil.error(logger,"创建充值订单接口失败");
      return null;
    }
    JSONObject resultJson = JSONObject.parseObject(result);
    String code = resultJson.getString("code");
    String orderNo = resultJson.getString("order_no");
    JSONObject data = resultJson.getJSONObject("pay_info");
    String orderstatus = resultJson.getString("orderstatus");
    if(REQ_STATUS_SUCCESS.equalsIgnoreCase(code)){
      //保存订单记录
      PayOrder payOrder = buildPayOrder(params);
      payOrder.setOrderId(resultJson.getString("shop_order_no"));
      payOrder.setOrderNo(orderNo);
      payOrder.setCustId(custId);
      payOrder.setCode(code);
      //logger.info("++++++++++++++++++++++++++++++++order type:"+OrderTypeEnum.ORDER_MERCH_RECHARGE.getValue().intValue());
      //payOrder.setOrderType(OrderTypeEnum.ORDER_MERCH_RECHARGE.getValue());
      if(!StringUtils.isBlank(refType) && refType.equalsIgnoreCase(String.valueOf(PayOrderTypeEnum.ORDER_MERCH_RECHARGE.getValue()))){
        payOrder.setOrderType(PayOrderTypeEnum.ORDER_MERCH_RECHARGE.getValue()); //商户账单支付
        payOrder.setOrderTypeRefId(refId);
      }
      payOrder.setOrderstatus(orderstatus);
      boolean ret = payOrderService.save(payOrder);
      if(ret){
        return data;
      }else{
        LogUtil.error(logger,"充值订单数据库保存失败.");
      }
    }
    return null;
  }

  @Override
  @Async
  public void payOrderStatusHandle(String orderNo) {
    //查询统一支付订单表，查询订单类型，然后根据不同类型做对应的逻辑处理
    //查询支付状态为1：支付成功，2：订单创建成功，等待支付。
    LambdaQueryWrapper<PayOrder> wrapper = new LambdaQueryWrapper();
    wrapper.eq(PayOrder::getOrderId,orderNo);
    wrapper.eq(PayOrder::getOrderstatus,"1");
    List<PayOrder> payOrderList = payOrderService.list(wrapper);
    if(payOrderList.size() != 1){
      //数据结构异常，告警
      LogUtil.error(logger,"数据结构异常，违反唯一约束，orderNo："+orderNo);
    }
    //商户充值订单
    PayOrder payOrder = payOrderList.get(0);
    logger.info("order type:"+payOrder.getOrderType());
    if (PayOrderTypeEnum.ORDER_MERCH_RECHARGE.getValue() == payOrder.getOrderType().intValue()) {
      // TODO 业务订单支付成功，调用业务订单服务处理
      // customerOrderService.paySuccess(payOrder);
    }
    else{
      //数据结构异常，告警
      LogUtil.error(logger,"该业务单据类型未支持，orderNo："+orderNo);
    }
}


  @Override
  public boolean drawings(String merchId, int money) {
    return false;
  }

  @Override
  public boolean drawingsApply(String merchId, int money) {
    return false;
  }

  @Override
  public boolean completeDrawingsApply(String merchId) {
    return false;
  }




  private PayOrder buildPayOrder(JSONObject jsonObject){
    PayOrder payOrder =  jsonObject.toJavaObject(PayOrder.class);
    return payOrder;
  }

  private JSONObject buildPayOrderReqData(String shopNo,String sign,int money,String openId){

    JSONObject sendData = new JSONObject();
    sendData.put("shop_no", shopNo);
    sendData.put("shop_order_no", String.valueOf(bizIdGenerator.newId()));
    sendData.put("shop_order_time", DateUtils.getTime());
    sendData.put("pay_resource", PAY_RESOURCE); //支付来源 1:支付宝  2:微信支付 5：云闪付
    sendData.put("trans_type_code",PAY_RESOURCE_TYPE); //1017：微信小程序
    sendData.put("transaction_amount", money);
    sendData.put("order_remark", "");
    sendData.put("timestamp", System.currentTimeMillis());
    sendData.put("callback_url", callbackUrl);

    sendData.put("order_name", "充值");
    sendData.put("sign", sign);
    sendData.put("client_ip", "113.31.163.91");

    sendData.put("auth_code", openId);
    sendData.put("notify_url", notifyUrl);
    sendData.put("app_id", appId);
    return sendData;
  }
}
